ui_config = {
    "rootPath": {
        "className": "QLineEdit",
        "data": "./cifs"
    },
    "choiceStructure": {
        "className": "QComboBox",
        "data": [],
        "currentIndex": -1
    },
    "savedPath": {
        "className": "QLineEdit",
        "data": "./saved"
    },
    "vectorA": {
        "className": "QComboBox",
        "data": [
            "1,0,0",
            "0,1,0",
            "1,1,0",
            "1,1,1",
            "-1,0,0",
            "0,-1,0",
            "1,-1,0",
            "-1,1,0",
            "-1,-1,0",
            "-1,-1,1",
            "-1,-1,-1"
        ],
        "currentIndex": 2
    },
    "vectorB": {
        "className": "QComboBox",
        "data": [
            "1,0,0",
            "0,1,0",
            "1,1,0",
            "1,1,1",
            "-1,0,0",
            "0,-1,0",
            "1,-1,0",
            "-1,1,0",
            "-1,-1,0",
            "-1,-1,1",
            "-1,-1,-1"
        ],
        "currentIndex": 6
    },
    "vectorC": {
        "className": "QComboBox",
        "data": [
            "0,0,1",
            "0,0,-1"
        ],
        "currentIndex": 0
    },
    "widthControlDelta": {
        "className": "QComboBox",
        "data": [
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
            "10"
        ],
        "currentIndex": 2
    },
    "widthControlOrigo": {
        "className": "QComboBox",
        "data": [
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
            "10"
        ],
        "currentIndex": 1
    },
    "widthControlBase": {
        "className": "QComboBox",
        "data": [
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
            "10"
        ],
        "currentIndex": 1
    },
    "ribbonTolerance": {
        "className": "QLineEdit",
        "data": "0.0"
    },
    "repeatA": {
        "className": "QLineEdit",
        "data": "3"
    },
    "repeatB": {
        "className": "QLineEdit",
        "data": "3"
    },
    "repeatC": {
        "className": "QLineEdit",
        "data": "1"
    },
    "minRibbonWidth": {
        "className": "QLineEdit",
        "data": "20"
    },
    "maxRibbonWidth": {
        "className": "QLineEdit",
        "data": "25"
    },
    "fixedRibbonWidth": {
        "className": "QLineEdit",
        "data": "0"
    },
    "vacuumA": {
        "className": "QLineEdit",
        "data": "0"
    },
    "vacuumB": {
        "className": "QLineEdit",
        "data": "15"
    },
    "vacuumC": {
        "className": "QLineEdit",
        "data": "15"
    },
    "connectivityMin": {
        "className": "QLineEdit",
        "data": "0"
    },
    "connectivityMax": {
        "className": "QLineEdit",
        "data": "1"
    },
    "bottom": {
        "className": "QLineEdit",
        "data": "0"
    },
    "top": {
        "className": "QLineEdit",
        "data": "15"
    },
    "ifWidthTest": {
        "className": "QCheckBox",
        "data": 0
    },
    "ifShowTargetStructure": {
        "className": "QCheckBox",
        "data": 1
    },
    "terminateAtom": {
        "className": "QComboBox",
        "data": [
            "",
            "H",
            "F",
            "Cl",
            "O"
        ],
        "currentIndex": 0
    }
}
